/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wBlowingResidual.h"
#include "wF0El.h"

#include "wElement.h"
#include "wCache.h"
#include "wGauss.h"

using namespace tbox;
using namespace dart;

/**
 * @brief Build the residual vector, on one boundary element
 *
 * b = \int psi * vn dV
 */
Eigen::VectorXd BlowingResidual::build(Element const &e, std::vector<double> const &phi, F0El const &f)
{
    // Get pre-computed values
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();

    // Blowing velocity
    double vn = f.eval(e, phi, 0);

    // Integral of (normal) transpiration velocity
    Eigen::VectorXd b = Eigen::VectorXd::Zero(e.nodes.size());
    for (size_t k = 0; k < gauss.getN(); ++k)
        b += cache.getSf(k) * vn * gauss.getW(k) * e.getDetJ(k);
    return b;
}

/**
 * @brief Build the gradient of the residual vector with respect to the blowing velocity, on one blowing boundary element
*/
Eigen::VectorXd BlowingResidual::buildGradientBlowing(tbox::Element const &e)
{
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();
    Eigen::VectorXd b = Eigen::VectorXd::Zero(e.nodes.size());
    for (size_t k = 0; k < gauss.getN(); ++k)
        b += cache.getSf(k) * gauss.getW(k) * e.getDetJ(k);
    return b;
}

/**
 * @brief Build the gradient of the residual vector with respect to the mesh, on one blowing boundary element
*/
Eigen::MatrixXd BlowingResidual::buildGradientMesh(tbox::Element const &e, std::vector<double> const &phi, F0El const &f, int const nDim)
{
    Eigen::MatrixXd A = Eigen::MatrixXd::Zero(e.nodes.size(), nDim * e.nodes.size());
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();
    // Blowing velocity
    double vn = f.eval(e, phi, 0);
    for (size_t k = 0; k < gauss.getN(); ++k)
    {
        std::vector<double> gradJ = e.getGradDetJ(k);
        for (size_t i = 0; i < gradJ.size(); i++)
            A.col(i) += cache.getSf(k) * vn * gauss.getW(k) * gradJ[i];
    }
    return A;
}
